#ifndef POLYNOME_HPP
#define POLYNOME_HPP
#include <cmath>
#include <iostream>
#include "complexe.hpp"
#include "rationnel.hpp"
using namespace std;

template <class T, class X> class polynome {
  int deg;
  X a;
  X b;
  T * coefs;
public: 
  polynome (){
    deg = 0;
    a = X(); b= X();
    coefs = NULL;
  }
  polynome (int degree){
    deg = degree;
    a = X(); b = X();
    coefs = new T[degree+1];
    for(int i =0; i<deg+1;i++){
      coefs[i] = T();
    }
  }
  polynome(int degree, X binf, X bsup){
    deg = degree;
    a = binf; b = bsup;
    coefs = new T[degree+1];
    for(int i =0; i<deg+1;i++){
      coefs[i] = T();
    }
  }
  polynome (const polynome & x){
    deg = x.deg;
    coefs = NULL;
    a = x.a; b = x.b;
    if(x.coefs != NULL){
      coefs = new T[deg+1];
      for(int i =0; i<deg+1;i++){
  	coefs[i] = x.coefs[i];
      }
    }
  }
  ~polynome (){
    if(coefs != NULL){
      delete [] coefs;
    }
  }
  T get(int);
  void set(T, int);
  polynome primitive();
  polynome derivative();
  X integral();
  polynome conjugue();
  polynome operator + (polynome);
  polynome operator - (polynome);
  polynome operator * (polynome);
  polynome & operator = (const polynome &);
  X operator , (polynome);
  X operator () (X);
  T operator [] (int);
  double norm2();
  bool operator <(polynome);
  bool operator >(polynome);
  bool operator ==(polynome);
  template <class U, class V>
  friend ostream & operator<<(ostream & out, const polynome<U,V> &);
  template <class U, class V>
  friend istream & operator>>(istream & in, polynome<U,V> &);
  int testu_d1();
  int testu_c1();
  int testu_i1();
  int all_testsu();

private:
  polynome convert(int deg2){
    if( deg2 < deg){
      cout<<"convertion not possible"<<endl;
    }
    polynome res(deg2);
    for(int i=0;i<deg+1;i++){
	res.coefs[i] = coefs[i];
    }
    return res;   
  }
  
};

template <class T, class X> T polynome<T,X>:: get(int i){
  T res = T();
  res= coefs[i];
  return res;
};

template <class T, class X> void polynome<T,X>:: set(T x, int i){
  coefs[i] = x;
};

template <class T, class X> polynome<T,X> polynome<T,X>:: operator + (polynome<T,X> p){
  int degt=0;
  degt = max(deg,p.deg);
  polynome res(degt,a,b), temp;
  
  if(deg < p.deg){ temp = this->convert(p.deg);
    for(int i=0;i<degt+1;i++){
      res.coefs[i] = temp.coefs[i] + p.coefs[i]; }
  }
  if(deg > p.deg){ temp = p.convert(deg); 
    for(int i =0;i<degt+1;i++){
      res.coefs[i]= coefs[i] + temp.coefs[i];  }
  }
  if(deg == p.deg){ 
    for(int i =0;i<degt+1;i++){
      res.coefs[i]= coefs[i] + p.coefs[i];  }
  }
  return res;
}  

template <class T, class X> polynome<T,X>  polynome<T,X>:: operator - (polynome<T,X> p){
  int degt=0;
  degt = max(deg,p.deg);
  polynome res(degt,a,b), temp;
  
  if(deg < p.deg){ temp = this->convert(p.deg);
    for(int i=0;i<degt+1;i++){
      res.coefs[i] = temp.coefs[i] - p.coefs[i];
    }
  }
  if(deg > p.deg){ temp = p.convert(deg); 
    for(int i =0;i<degt+1;i++){
      res.coefs[i]= coefs[i] - temp.coefs[i];
    }
  }
  if(deg > p.deg){
    for(int i =0;i<degt+1;i++){
      res.coefs[i]= coefs[i] - p.coefs[i];  }
  }
  return res;
}  


template <class T, class X> polynome<T,X>  polynome<T,X>:: operator * (polynome<T,X> p){
  int degt=deg+p.deg;
  polynome<T,X> res(degt,a,b);
  polynome<T,X> temp, temp2;

  temp = this->convert(degt);
  temp2 = p.convert(degt);
  for(int i=0;i<degt+1;i++){
    for(int j=0;j<i+1;j++){
      res.coefs[i]=res.coefs[i]+temp.coefs[j]*temp2.coefs[i-j];
    }
  }
  return res;
} 

template<class T,class X> polynome<T,X> & polynome<T,X>:: operator = (const polynome<T,X> & p){
  if(&p != this){
     deg = p.deg;
     a = p.a; b = p.b;
     if(coefs != NULL) { delete [] coefs; }
     coefs = NULL;
     if(p.coefs != NULL){
       coefs = new T[deg+1];
       for(int i =0;i<deg+1;i++){ coefs[i]=p.coefs[i]; } }
   }
   return *this;
 }

template <class T, class X> T polynome<T,X>::operator [](int i){
  return coefs[i];
}

template <class T, class X> X polynome<T,X>::operator ()(X x){
  T res =T();  T prod= T();
  res = res + coefs[0];
  for(int i =1;i<deg+1;i++){
    prod =x;
    for(int j=1;j<i;j++){ prod = prod*x; }
    res = res + coefs[i]*prod;
  }
  return res;
}

template <class T, class X> X polynome<T,X>:: operator , (polynome<T,X> p){
  polynome pro, prim;
  X res =X();
  pro = (*this)*p.conjugue();
  res = pro.integral();
  return res;
}

template <class T, class X> double polynome<T,X>::norm2(){
  double n = 0.0;
  n = (*this,*this);
  return n;
};

template <>
double polynome< complexe<double>,complexe<double> >::norm2(){
  double n = 0.0;
  n = (*this,*this).module();
  return n;
};

template <class T, class X> bool polynome<T,X>:: operator < (polynome<T,X> x){
  bool res = false;
  if(this->norm2() < x.norm2() ) { res = true;}
  return res;
};

template <class T, class X> bool polynome<T,X>:: operator > (polynome<T,X> x){
  bool res = false;
  if(this->norm2() > x.norm2()) { res = true;}
  return res;
};

template <class T, class X> bool polynome<T,X>:: operator == (polynome<T,X> x){
  bool res = false;
  if(this->norm2() == x.norm2()) { res = true;}
  return res;
};

template <class T, class X> polynome<T,X> polynome<T,X>::conjugue() {
  return *this;
}

template<>
polynome< complexe<double>, complexe<double> > polynome< complexe<double>, complexe<double> >::conjugue(){
  polynome< complexe<double>, complexe<double> > res(deg);
  for(int i=0;i<deg+1;i++){ res.coefs[i]=coefs[i].conjuge();  }
  return res;
}

template<>
polynome< complexe<int>, complexe<int > > polynome< complexe<int>, complexe<int> >::conjugue(){
  polynome< complexe<int>, complexe<int> > res(deg);
  for(int i=0;i<deg+1;i++){
    res.coefs[i]=coefs[i].conjuge();
  }
  return res;
}


template <class T, class X> polynome<T,X> polynome<T,X>::derivative(){
  polynome<T,X> res(deg-1,a,b);
  for(int i =1;i<deg+1;i++){
    res.coefs[i-1]= i*coefs[i];
  }
  return res;
};

template <class T, class X> polynome<T,X> polynome<T,X>::primitive(){
  polynome<T,X> res(deg+1,a,b);
  for(int i=0;i<deg+1;i++){
    res.coefs[i+1]= (1.0/(i+1.0))*coefs[i];
  }
  res.coefs[0]=T();
  return res;
}

template <class T, class X> X polynome<T,X>::integral(){
  X res = X();
  polynome<T,X> primitive;
  primitive = this->primitive();
  res= primitive(b)-primitive(a);
  return res;
}

template <class T, class X> ostream & operator << (ostream & out,const polynome<T,X> & p){
  for(int i=0;i<p.deg+1;i++){
    out<<"("<<p.coefs[i]<<")"<<"x^"<<i;
    if(i!= p.deg){ out<<" + ";}
  }
  return out;
}


template <class T, class X> istream & operator >>(istream & in, polynome<T,X> & p){
  for(int i=0;i<p.deg+1;i++){
    cout<<" coef: "<<i<<endl;
    in >> p.coefs[i];
  }
  return in;
}

template<>
int polynome<double,double>:: testu_d1(){
  polynome<double,double> P(2);
  polynome<double,double>  Q,R,S;

   *this = P;
   this->set(1.0,0); this->set(2.0,1); this->set(1.0,2);
   Q = this->derivative();
   R = this->primitive();
   S = *this+R;
   *this = S*Q;
   if((*this)[0] == 2.0 && (*this)[2] == 10.0){ return 0;}
   else { return 1;}
  return 1;
}

template<>
int polynome< complexe<rationnel> , complexe<rationnel> >:: testu_c1(){
  polynome< complexe<rationnel> , complexe<rationnel> > P(2);
  polynome< complexe<rationnel> , complexe<rationnel> >  Q,R,S;
  complexe<rationnel> x(rationnel(1,1),rationnel(1,1));
  complexe<rationnel> y(rationnel(2,1),rationnel(-1,1));
  complexe<rationnel> z(rationnel(1,1),rationnel(0,1));

  *this = P;
  this->set(x,0); this->set(y,1); this->set(z,2);
  Q = this->derivative();
  R = this->primitive();
  S = *this+R;
  *this = S*Q;
  
  if((*this)[1].reel().num() == 28 && (*this)[1].img().num() == -8){ return 0;}
  else { return 1;}
}

template<>
int polynome<double , double>:: testu_i1(){
    polynome <double , double> P(1,0,1), Q(1,0,1);
    double normeP, normeQ;
    
    *this = P;
    this->set(1.0,0); this->set(2.0,1);
    normeP = this->norm2();
    *this = Q;
    this->set(2.0,0); this->set(-1.0,1);
    normeQ = this->norm2();
    if(abs(normeP - 4.33333333333333) < 1e-8 && abs(normeQ - 2.3333333333333) < 1e-8){ return 0;}
    else { return 1;}
}

template <class T, class X> int polynome<T,X>:: all_testsu(){
  int c=0,t=0;
  polynome<double,double> p1;
  c=p1.testu_d1(); t = t +c;
  c=p1.testu_i1(); t = t +c;
  polynome< complexe<rationnel> , complexe<rationnel> > p2;
  c=p2.testu_c1(); t = t +c;

  cout<<"Tests unitaires réussis, classe polynome: "<<3-t<<"/3"<<endl;
  return 0;
}


#endif
